' ********** Copyright 2018 Roku, Inc.  All Rights Reserved. ********** 
Library "Roku_Ads.brs"

function init()
   '
   '  Following info must be provided
   '
   '   m.top.testConfig.url          :  master URL
   '   m.top.testConfig.params       :  optional object, adsParams of initial request
   '   m.top.testConfig.useStitched  :  true or false
   '
    m.top.adPlaying = False
    m.top.functionName = "runTask"
    m.useStitched = true
    m.adPods = invalid
end function

function runTask()
    '
    '  1. Load and instanciate Adopter
    '
    adapter = loadAdapter()
    '
    '  2. Request preplay
    '
    loadStream(adapter)
    '
    '  3. Play content
    '
    runLoop(adapter)
end function

function loadAdapter() as object
    '  not calling callStitchedAdHandledEvent(). App is responsible to call RAF.fireTrackingEvents()
    if invalid <> m.top.testConfig["useStitched"] and false = m.top.testConfig.useStitched then m.useStitched = false
    '
    '  1. Load and instanciate Adopter
    '
    adapter = RAFX_SSAI({name:"ggldai"}) ' Required
    if adapter <> invalid
        adapter.init() ' Required
        print "RAFX_SSAI version ";adapter["__version__"]
    end if
    return adapter
end function

function loadStream(adapter as object) as void
    if invalid = adapter then return
    '
    '  2.1 Compose request info
    '
    request = {}
    if invalid <> m.top.testConfig["assetKey"]
        request["type"] = adapter.StreamType.LIVE
        request["assetKey"] = m.top.testConfig["assetKey"]
    else if invalid <>  m.top.testConfig["contentSourceId"]
        request["type"] = adapter.StreamType.VOD
        request["contentSourceId"] = m.top.testConfig["contentSourceId"]
        request["videoId"] = m.top.testConfig["videoId"]
    end if
    requestResult = adapter.requestStream(request) ' Required, requesting boot-strap JSON
    if requestResult["error"] <> invalid
        print "Error requesting stream ";requestResult
    else
        '
        '  2.2  Setup video node.  Select here variant
        '
        streamInfo = adapter.getStreamInfo() ' Required when vod, optional when live
        vidContent = createObject("RoSGNode", "ContentNode")
        vidContent.url = streamInfo["playURL"]
        vidContent.title = m.top.testConfig.title
        vidContent.streamformat = "hls"
        m.top.video.content = vidContent
        m.top.video.setFocus(true)
        m.top.video.visible = true
        m.top.video.enableCookies()
    end if
    '
    '   2.4  other RAF settings
    '
    adIface = Roku_Ads()
    adIface.enableAdMeasurements(true) ' Required
    ' adIface.setContentLength()    ' Set app/content specific info
    ' adIface.setNielsenProgramId() ' Set app/content specific info
    ' adIface.setNielsenGenre()     ' Set app/content specific info
    ' adIface.setNielsenAppId()     ' Set app/content specific info
    ' adIface.setDebugOutput(true)
end function

function runLoop(adapter as object) as void
    if invalid = adapter then return
    '
    '   3.1  Set callbacks
    '
    addCallbacks(adapter) ' Required when not using RAF.stitchedAdHandledEvent()
    ' 
    '   3.2  Enable adapter Ad tracking
    ' 
    port = CreateObject("roMessagePort")
    '
    '  Option:  RAFX_SSAI() adapter provides options to use RAF
    '    A. use RAF (default)
    '       When AdPod found, calls stitchedAdsInit()
    '       When msg, calls stitchedAdHandledEvent()
    '       this option allows both video and interactive ads rendered by RAF
    '    B. no RAF.stitchedAd call
    '       App is responsible to fire pixel via RAF.fireTrackingEvents()
    '       See callback functions in this sample
    '
    adapter.enableAds({ player : {sgnode:m.top.video, port:port},              ' Required
                        useStitched : m.useStitched  ' Optional, default is true
                     })
    '
    '   3.3  Observe video node
    '
    m.top.video.observeField("position", port) ' Required
    m.top.video.observeField("control", port)
    m.top.video.observeField("state", port)
    '
    '   3.4  Start playback and fire uo:contentImpression
    '
    m.top.video.control = "play"
    m.lastPosition = -1
    while true
        msg = wait(1000, port)
        if type(msg) = "roSGNodeEvent" and msg.getField() = "control" and msg.getNode() = m.top.video.id and not m.top.adPlaying and msg.getData() = "stop" or m.top.video = invalid
            exit while  ' video node stopped. quit loop
        end if
        '
        '  3.5  Have adapter handle events
        '
        currentTime = m.top.video.position
        if isSeeked(currentTime, adapter) ' Optional, snap seek time to Begin/End of Pod
            m.lastPosition = -1
        else
            m.lastPosition = currentTime
            curAd = adapter.onMessage(msg) ' Required
            if invalid = curAd
                m.top.video.setFocus(true)
                if type(msg) = "roSGNodeEvent" and not m.top.adPlaying
                    if msg.getField() = "state" and msg.getData() = "finished" and msg.getNode() = m.top.video.id then ' stream ended. quit loop
                        exit while
                    end if
                end if
            else
                if invalid <> m.adPods
                    adPod = m.adPods[curAd["adPodIndex"]-1]
                end if
            end if
        end if
    end while
end function

function isSeeked(currentTime, adapter) as boolean
    seekSecThreshold = 3
    '
    ' Optional, applicable to VOD mode
    ' When time difference is above threshold, assume video is rewinding/fastfowarding, and options are
    '     SnapTo.BEGIN.
    '        When seeked position is between pod.begin and pod.end, snap to beginning of pod
    '        so that Ads will play
    '     SnapTo.END
    '        When seeked position is between pod.begin and pod.end, snap to end of pod
    '        so that Ads will not play
    '
    if 0 < m.lastPosition and seekSecThreshold < abs(currentTime - m.lastPosition)
        currentTime = int(currentTime)
        snapTime = adapter.onSeekSnap({time:currentTime, snapto:adapter.SnapTo.BEGIN}) ' snap to pod BEGIN or END
        if snapTime <> currentTime
            print "task.isSeeked() seek Time snapped from: ";currentTime;"  to: ";snapTime
            m.top.video.seek = snapTime
        end if
        return true
    end if
    return false
end function

'
'   Required, Configure callback functions when NOT using RAF.stitchedAdHandledEvent()
'
function addCallbacks(adapter) as void
    adapter.addEventListener(adapter.AdEvent.PODS, podsCallback)
    adapter.addEventListener(adapter.AdEvent.POD_START, podStartCallback)
    adapter.addEventListener(adapter.AdEvent.IMPRESSION, adEventCallback)
    adapter.addEventListener(adapter.AdEvent.FIRST_QUARTILE, adEventCallback)
    adapter.addEventListener(adapter.AdEvent.MIDPOINT, adEventCallback)
    adapter.addEventListener(adapter.AdEvent.THIRD_QUARTILE, adEventCallback)
    adapter.addEventListener(adapter.AdEvent.COMPLETE, adEventCallback)
    adapter.addEventListener(adapter.AdEvent.POD_END, podEndCallback)
    '
    m.adPod = invalid
    m.adIndex = 0
    m.COMPLETE = adapter.AdEvent.COMPLETE
end function

function podsCallback(podsInfo as Object)
    adPods = podsInfo["adPods"] ' New list of adPods found
    print " Pod count: ";adPods.count()
    for each adPod in adPods
        print "     Ad count: ";adPod.ads.count(); "  renderTime: ";adPod.renderTime; "  endTime: ";adPod.renderTime+adPod.duration
    end for
    m.adPods = adPods
end function
function podStartCallback(podInfo as Object)
    if not m.top.adPlaying
        m.top.adPlaying = True
        m.top.video.enableTrickPlay = false
    end if
    if not m.useStitched
        m.adPod = podInfo["adPod"]
        if invalid <> m.adPod
            print "  Pod adCount: ";podInfo["adPod"].ads.count()
            adIface = Roku_Ads()
            ' fire Pod pixel
            adIface.fireTrackingEvents(m.adPod, {type: podInfo.event})
            m.adIndex = 0
        end if
    end if
end function
function adEventCallback(adInfo as Object) as Void
    print "At ";adInfo.position;" from SDK -- " ; adInfo.event
    if invalid <> m.adPod and m.adIndex < m.adPod.ads.count()
        adIface = Roku_Ads()
        ' fire Ad pixel
        ad = m.adPod.ads[m.adIndex]
        adIface.fireTrackingEvents(ad, {type: adInfo.event})
        if m.COMPLETE = adInfo.event
            m.adIndex += 1
        end if
    end if
end function
function podEndCallback(podInfo as Object)
    m.top.adPlaying = False
    m.top.video.enableTrickPlay = true
    m.top.video.setFocus(true)
    if invalid <> m.adPod
        adIface = Roku_Ads()
        ' fire Pod pixel
        adIface.fireTrackingEvents(m.adPod, {type: podInfo.event})
        m.adIndex = 0
    end if
    m.adPod = invalid
end function
